@@ -13,6 +13,13 @@ except ImportError:
 linestyles= ["-", "--", ":", "-."]
 
 
+def centeroidnp(arr):
+    length = arr.shape[0]
+    sum_x = np.sum(arr[:, 0])
+    sum_y = np.sum(arr[:, 1])
+    return sum_x/length, sum_y/length
+
+
 class UndefinedUniverseOfDiscourseError(Exception):
 
 	def __init__(self, message):
@@ -21,7 +28,7 @@ class UndefinedUniverseOfDiscourseError(Exception):
 
 class LinguisticVariable(object):
 
-	def __init__(self, FS_list=[], concept="", universe_of_discourse=None):
+	def __init__(self, FS_list=[], concept=None, universe_of_discourse=None):
 		"""
 		Creates a new linguistic variable.
 		Args:
@@ -34,9 +41,9 @@ class LinguisticVariable(object):
 		if FS_list==[]:
 			print("ERROR: please specify at least one fuzzy set")
 			exit(-2)
-		if concept=="":
-			print("ERROR: please specify a concept connected to the linguistic variable")
-			exit(-3)
+		#if concept is None:
+		#	print("ERROR: please specify a concept connected to the linguistic variable")
+		#	exit(-3)
 		self._universe_of_discourse = universe_of_discourse
 
 		self._FSlist = FS_list
@@ -182,6 +189,8 @@ class FuzzySystem(object):
 			LV: linguistic variable object to be added to the fuzzy system.
 			verbose: True/False, toggles verbose mode.
 		"""
+		if LV._concept is None: 
+			LV._concept = name
 		self._lvs[name]=LV
 		if verbose: print(" * Linguistic variable '%s' successfully added" % name)
 
@@ -209,7 +218,7 @@ class FuzzySystem(object):
 		self._outputfunctions[name]=function
 		if verbose: print(" * Output function for '%s' set to '%s'" % (name, function))
 
-	def set_output_FS(self, name, fuzzyset, verbose=False):
+	def set_output_FS(self, fuzzyset, verbose=False):
 		"""
 		Adds a new output function as a Fuzzy Set object.
 		Args: 
@@ -217,7 +226,7 @@ class FuzzySystem(object):
 			fuzzyset: FuzzySet object used as output (in a Mamdani FIS).
 			verbose: True/False, toggles verbose mode.
 		"""
-		self._outputfuzzysets[name]=fuzzyset
+		self._outputfuzzysets[fuzzyset.get_term()]=fuzzyset
 		if verbose: print(" * Output fuzzy set for '%s' set" % (name))
 
 
@@ -299,8 +308,7 @@ class FuzzySystem(object):
 				outname = res[0]
 				outterm = res[1]
 
-				if verbose:
-					print(ant,res,outname,outterm)			
+				if verbose:	print(ant,res,outname,outterm)			
 
 				if outname==output:
 
@@ -314,7 +322,9 @@ class FuzzySystem(object):
 					cuts_list[outterm] = value
 
 			values = []
+			weightedvalues = []
 			integration_points = linspace(x0, x1, subdivisions)
+
 			for u in integration_points:
 				#print ("x=%.1f" % u)
 				comp_values = []
@@ -323,9 +333,12 @@ class FuzzySystem(object):
 					comp_values.append(result)
 				keep = max(comp_values)
 				values.append(keep)
+				weightedvalues.append(keep*u)
 
-			cog = sum(values)/subdivisions
-			final_result[output] = cog
+			CoG = sum(weightedvalues)/sum(values)
+			if verbose: print (" * CoG:", CoG)
+			
+			final_result[output] = CoG 
 
 		return final_result
 
@@ -376,7 +389,6 @@ class FuzzySystem(object):
 		return result
 
 
-
 	def produce_figure(self, outputfile='output.pdf'):
 		"""
 		Plots the membership functions of each linguistic variable contained in the fuzzy system.
